#pragma once

#include <fstream>
#include <sstream>
#include <iostream>
#include <ctime>
#include <vector>
#include <list>
#include <deque>
#include <algorithm>

static void CreateDataStream(const std::string& filename) {
    std::ofstream file(filename.c_str());
    for (int i = 0; i < 100; i++)
        file << -50 + std::rand() % 101 << " ";
    file.close();
}

static void CreateDataGenerate(const std::string& filename) {
    std::ofstream file(filename.c_str());
    std::vector<int> data(100);
    std::generate(data.begin(), data.end(), [] { return -50 + std::rand() % 101; });
    std::copy(data.begin(), data.end(), std::ostream_iterator<int>(file, " "));
    file << std::endl;
    file.close();
}

template<typename Container>
Container LoadDataStream(const std::string& filename) {
    Container cont;
    std::ifstream fin(filename);
    typename Container::value_type x;
    while (fin >> x)
        cont.push_back(x);
    return cont;
}

template<typename Container>
Container LoadDataIterator(const std::string& filename) {
    std::ifstream file(filename.c_str());
    Container cont(100);
    using value_type = typename Container::value_type;
    std::copy(std::istream_iterator<value_type>(file), std::istream_iterator<value_type>(), cont.begin());
    file.close();
    return cont;
}

template<typename Container>
void ModifyTemplate(Container &src) {
    Container result;
    auto min = *std::min_element(src.begin(), src.end());
    auto max = *std::max_element(src.begin(), src.end());
    for (auto &el: src) {
        el = el / min + max;
    }
}

template<typename Iter>
void ModifyIter(const Iter src_begin, const Iter src_end) {
    auto min = *std::min_element(src_begin, src_end);
    auto max = *std::max_element(src_begin, src_end);
    for (auto it = src_begin; it != src_end; ++it)
        *it = *it / min + max;
}

template<typename Container>
void ModifyForEach(Container &src) {
    auto min = *std::min_element(src.begin(), src.end());
    auto max = *std::max_element(src.begin(), src.end());
    auto modify = [min, max](auto &el) {
        el = el / min + max;
        return el;
    };
    std::for_each(src.begin(), src.end(), modify);
}

static void ModifyTransform(const std::string& in_name, const std::string& out_name) {
    std::ifstream fin(in_name.c_str());
    std::ofstream fout(out_name.c_str());
    std::vector<float> vec(100);
    std::copy(std::istream_iterator<float>(fin), std::istream_iterator<float>(), vec.begin());
    auto min = *std::min_element(vec.begin(), vec.end());
    auto max = *std::max_element(vec.begin(), vec.end());
    auto modify = [min, max](auto &el) {
        el = el / min + max;
        return el;
    };
    std::transform(vec.begin(), vec.end(), std::ostream_iterator<float>(fout, " "), modify);
    fout << std::endl;
    fin.close();
    fout.close();
}

template<typename Container>
void OutputResultStream(const std::string& out_filename, const Container &src) {
    std::ofstream fout(out_filename.c_str());
    for (const auto &el: src)
        fout << el << " ";
    fout << std::endl;
    fout.close();
}

template<typename Container>
void OutputResultCopy(const std::string& out_filename, const Container &src) {
    using value_type = typename Container::value_type;
    std::ofstream fout(out_filename.c_str(), std::ios::app);
    std::copy(src.begin(), src.end(), std::ostream_iterator<value_type>(fout, " "));
    fout << std::endl;
    fout.close();
}

template<typename Container>
void PrintContainer(const Container& cont) {
    for (auto i : cont) {
        std::cout << i << " ";
    }
}

struct Application {

    Application() = default;

    void run() {
        std::srand(std::time(nullptr));
        bool is_running = true;
        int option;
        while (is_running) {
            std::cout << std::endl << "Choose container:" << std::endl
                      << "1)Vector" << std::endl
                      << "2)List" << std::endl
                      << "3)Deque" << std::endl
                      << "4)Exit" << std::endl << ">>>";
            std::cin >> option;
            std::cout << std::endl;
            std::deque<float> deq;
            std::list<float> lst;
            std::vector<float> vec;
            switch (option) {
                case 1:
                    main_menu(deq);
                    deq.clear();
                    break;
                case 2:
                    main_menu(lst);
                    lst.clear();
                    break;
                case 3:
                    main_menu(vec);
                    vec.clear();
                    break;
                case 4:
                    is_running = false;
                    break;
                default:
                    std::cout << "invalid option" << std::endl;
                    break;
            }
        }
    }

    template <typename Container>
    void main_menu(Container& cont) {
        std::string filename("file.txt");
        std::string out_filename("file_out.txt");
        bool is_running = true;
        int option;
        while (is_running) {
            std::cout << std::endl << "Choose func:" << std::endl
                      << "1)CreateDataStream" << std::endl
                      << "2)CreateDataIterator" << std::endl
                      << "3)LoadDataStream" << std::endl
                      << "4)LoadDataIterator" << std::endl
                      << "5)PrintContainer" << std::endl
                      << "6)ModifyTemplate" << std::endl
                      << "7)ModifyIter" << std::endl
                      << "8)ModifyForEach" << std::endl
                      << "9)ModifyTransform" << std::endl
                      << "10)OutputResultStream" << std::endl
                      << "11)OutputResultCopy" << std::endl
                      << "12)Choose other container" << std::endl << ">>>";
            std::cin >> option;
            std::cout << std::endl;
            switch (option) {
                case 1:
                    CreateDataStream(filename);
                    std::cout << "OK" << std::endl;
                    break;
                case 2:
                    CreateDataGenerate(filename);
                    std::cout << "OK" << std::endl;
                    break;
                case 3:
                    cont = std::move(LoadDataStream<Container>(filename));
                    std::cout << "OK" << std::endl;
                    break;
                case 4:
                    cont = std::move(LoadDataIterator<Container>(filename));
                    std::cout << "OK" << std::endl;
                    break;
                case 5:
                    PrintContainer(cont);
                    break;
                case 6:
                    ModifyTemplate(cont);
                    std::cout << "OK" << std::endl;
                    break;
                case 7:
                    ModifyIter(cont.begin(), cont.end());
                    std::cout << "OK" << std::endl;
                    break;
                case 8:
                    ModifyForEach(cont);
                    std::cout << "OK" << std::endl;
                    break;
                case 9:
                    ModifyTransform(filename, out_filename);
                    std::cout << "OK" << std::endl;
                    break;
                case 10:
                    OutputResultStream(out_filename, cont);
                    std::cout << "OK" << std::endl;
                    break;
                case 11:
                    OutputResultCopy(out_filename, cont);
                    std::cout << "OK" << std::endl;
                    break;
                case 12:
                    is_running = false;
                    break;
                default:
                    std::cout << "Wrong input" << std::endl;
                    break;
            }
        }
    }
};
