#include "File.h"
#include <utility>
#include "iostream"
#include "fstream"
#include "iomanip"
#include "algorithm"

File::File(std::string fname) : fname(std::move(fname)) {}

void File::GetData() {
    std::ifstream fin(fname);
    Record rec;
    int ind = 0;
    while (fin.read(reinterpret_cast<char *>(&rec), sizeof(Record))) {
        ind++;
        rec_arr.insert({ind, rec});
    }
}

void File::AddToFile(const Record &rec) {
    std::ofstream fout(fname, std::ios::binary | std::ios::app);
    fout.write(reinterpret_cast<const char *>(&rec), sizeof(Record));
    fout.close();
}

void File::DelFromFile(int ind) {
    GetData();
    rec_arr.erase(ind);
    std::ofstream fout(fname + "_deleted", std::ios::binary);
    for (const auto &elem: rec_arr) {
        fout.write(reinterpret_cast<const char *>(&elem.second), sizeof(Record));
    }
    fout.close();
    File(fname + "_deleted").ToText();
}

// in-place замена

void File::Replace(int ind, const Record &rec) {
    GetData();
    std::fstream fout(fname,  std::ios::binary | std::ios::in | std::ios::out);
    rec_arr[ind] = rec;
    if (ind >= 1 && ind <= rec_arr.size()) {
        fout.seekp((ind - 1) * sizeof(Record));
        fout.write(reinterpret_cast<const char *>(&rec), sizeof(Record));
    } else
        std::cout << "Cannot replace!" << std::endl;
    fout.close();
    File(fname).ToText();
}

void File::ToText() {
    GetData();
    std::ofstream fout(fname + ".txt");
    fout << "|   "
         << "|" << std::setw(13) << "House #"
         << "|" << std::setw(10) << "Flat #"
         << "|" << std::setw(10) << "Sec name"
         << "|" << std::setw(10) << "Oper tp"
         << "|" << std::setw(10) << "Date"
         << "|" << std::setw(10) << "Oper sum"
         << "|" << std::setw(10) << "Penalt"
         << "|" << std::setw(10) << "Delay"
         << "|" << std::endl;

    int cnt = 0;
    for (const auto &elem: rec_arr) {
        cnt++;
        fout << "|" << std::setw(3) << cnt << "|"
             << std::setw(13) << elem.second.houseN << "|"
             << std::setw(10) << elem.second.flatN << "|"
             << std::setw(10) << elem.second.secondName << "|"
             << std::setw(10) << elem.second.operType << "|"
             << std::setw(10) << elem.second.date << "|"
             << std::setw(10) << elem.second.operSum << "|"
             << std::setw(10) << elem.second.penalties << "|"
             << std::setw(10) << elem.second.delay << "|"
             << std::endl;
    }
    fout.close();
}

void File::SearchByHouseN(int r_houseN) {
    GetData();
    std::ofstream fout(std::to_string(r_houseN) + "_house", std::ios::binary);
    std::for_each(rec_arr.begin(), rec_arr.end(), [&r_houseN, &fout](const std::pair<int, Record> &elem) {
        if (elem.second.houseN == r_houseN)
            fout.write(reinterpret_cast<const char *>(&elem.second), sizeof(Record));
    });
    fout.close();
    File(std::to_string(r_houseN) + "_house").ToText();
}

void File::SearchByFlatN(int r_flatN) {
    GetData();
    std::ofstream fout(std::to_string(r_flatN) + "_flat", std::ios::binary);
    std::for_each(rec_arr.begin(), rec_arr.end(), [&r_flatN, &fout](const std::pair<int, Record> &elem) {
        if (elem.second.flatN == r_flatN)
            fout.write(reinterpret_cast<const char *>(&elem.second), sizeof(Record));
    });
    fout.close();
    File(std::to_string(r_flatN) + "_flat").ToText();
}

void File::SearchBySecondName(const std::string &r_secondName) {
    GetData();
    std::ofstream fout(r_secondName + "_owner", std::ios::binary);
    std::for_each(rec_arr.begin(), rec_arr.end(), [&r_secondName, &fout](const std::pair<int, Record> &elem) {
        if (elem.second.secondName == r_secondName)
            fout.write(reinterpret_cast<const char *>(&elem.second), sizeof(Record));
    });
    fout.close();
    File(r_secondName + "_owner").ToText();
}

void File::SearchByOperType(const std::string &r_operType) {
    GetData();
    std::ofstream fout(r_operType + "_operation", std::ios::binary);
    std::for_each(rec_arr.begin(), rec_arr.end(), [&r_operType, &fout](const std::pair<int, Record> &elem) {
        if (elem.second.operType == r_operType)
            fout.write(reinterpret_cast<const char *>(&elem.second), sizeof(Record));
    });
    fout.close();
    File(r_operType + "_operation").ToText();
}

void File::SearchByDate(const std::string &r_date) {
    GetData();
    std::ofstream fout(r_date + "_date", std::ios::binary);
    std::for_each(rec_arr.begin(), rec_arr.end(), [&r_date, &fout](const std::pair<int, Record> &elem) {
        if (elem.second.date == r_date)
            fout.write(reinterpret_cast<const char *>(&elem.second), sizeof(Record));
    });
    fout.close();
    File(r_date + "_date").ToText();
}


